// Viewport Helper
// --------------------------------------------
//
// * **Class:** Viewport
// * **Version:** 1.0
// * **Modified:** 06/26/2013
// * **Author:** Glen Cheney
// * **Dependencies:** jQuery 1.7+, SONY Settings, throttle/debounce
//
// *Notes:*
//
// If you need to be notified when an element is scrolled into view, use this module.
// This module keeps track of all elements that want to be watched and caches their offsets
// and dimensions in order to keep scrolling as smooth as possible.
//
// *Example Usage:*
//
//      Viewport.add({
//        element: document.getElementById('some-wrapper'),
//        threshold: '50%',
//        enter: function( element ) {
//          console.log('the top of "element" is 50% in view');
//        },
//        leave: function() {
//          console.log('bottom of element has left the viewport');
//        }
//      });
//
// *Viewport.add parameters:*
//
// * `element` is a DOM element and `callback` is a function. `this` in the callback is the element.
// * Using an options object, a `threshold` can be set.
//   It is either an integer value from the bottom of the window, a string percentage, or a float
//   between 0 and 1 which represents the percent.
//
(function(e){"use strict";var t=null,n=e(window),r=function(t){var n=this;e.extend(n,r.options,t,r.settings);if(!e.isFunction(n.enter))throw new TypeError("Viewport.add :: No `enter` function provided in Viewport options.");if(typeof n.threshold=="string"&&n.threshold.indexOf("%")>-1){n.isThresholdPercentage=!0;n.threshold=parseFloat(n.threshold)/100}else n.threshold<1&&n.threshold>0&&(n.isThresholdPercentage=!0);n.hasLeaveCallback=e.isFunction(n.leave);n.$element=e(n.element);n.update()};r.prototype.update=function(){var e=this;e.offset=e.$element.offset();e.height=e.$element.height();e.width=e.$element.width()};r.options={threshold:200,delay:0};r.settings={triggered:!1,isThresholdPercentage:!1};var i=function(){this.init()};i.prototype={init:function(){var e=this;e.list=[];e.lastScrollY=0;e.windowHeight=n.height();e.windowWidth=n.width();e.throttleTime=100;e.onResize();e.bindEvents();e.willProcessNextFrame=!0;requestAnimationFrame(function(){e.setScrollTop();e.process();e.willProcessNextFrame=!1})},bindEvents:function(){var t=this,r;r=function(){setTimeout(function(){t.refresh()},0)};n.on("resize.viewport",e.proxy(t.onResize,t));n.on("scroll.viewport",e.throttle(t.throttleTime,e.proxy(t.onScroll,t)));t.hasActiveHandlers=!0},unbindEvents:function(){n.off(".viewport");this.hasActiveHandlers=!1},maybeUnbindEvents:function(){var e=this;e.list.length||e.unbindEvents()},add:function(e){var t=this;t.list.push(e);t.hasActiveHandlers||t.bindEvents();if(!t.willProcessNextFrame){t.willProcessNextFrame=!0;requestAnimationFrame(function(){t.willProcessNextFrame=!1;t.process()})}},saveDimensions:function(){var t=this;e.each(t.list,function(e,t){t.update()});t.windowHeight=n.height();t.windowWidth=n.width()},onScroll:function(){var e=this;if(!e.list.length)return;e.setScrollTop();e.process()},onResize:function(){this.refresh()},refresh:function(){if(!this.list.length)return;this.saveDimensions()},isInViewport:function(e){var t=this,n=e.offset,r=e.threshold,i=r,s=t.lastScrollY,o;e.isThresholdPercentage&&(r=0);o=t.isTopInView(s,t.windowHeight,n.top,e.height,r);o&&e.isThresholdPercentage&&(o=t.isTopPastPercent(s,t.windowHeight,n.top,e.height,i));return o},isTopInView:function(e,t,n,r,i){var s=e+t;return n+i>=e&&n+i<s},isTopPastPercent:function(e,t,n,r,i){var s=e+t,o=s-n,u=o/t;return u>=i},isOutOfViewport:function(e,t){var n=this,r=e.offset,i=n.lastScrollY,s;t==="bottom"&&(s=!n.isBottomInView(i,n.windowHeight,r.top,e.height));return s},isBottomInView:function(e,t,n,r){var i=e+t,s=n+r;return s>e&&s<=i},triggerEnter:function(t){var n=this;setTimeout(function(){t.enter.call(t.element,t)},t.delay);e.isFunction(t.leave)?t.triggered=!0:n.list.splice(e.inArray(t,n.list),1);n.maybeUnbindEvents()},triggerLeave:function(e){setTimeout(function(){e.leave.call(e.element,e)},e.delay);e.triggered=!1},setScrollTop:function(){this.lastScrollY=n.scrollTop()},process:function(){var t=this,n=e.extend([],t.list);e.each(n,function(e,n){var r=t.isInViewport(n),i=n.hasLeaveCallback&&t.isOutOfViewport(n,"bottom");if(!n.triggered&&r)return t.triggerEnter(n);if(!r&&i&&n.triggered)return t.triggerLeave(n)})}};i.add=function(e){var t=i.getInstance();return t.add(new r(e))};i.refresh=function(){i.getInstance().refresh()};i.getInstance=function(){t||(t=new i);return t};window.Viewport=i})(jQuery);